@@ -69,11 +69,6 @@ public:
 	ZLIntegerRangeOption KeyScrollingDelayOption;
 	ZLIntegerRangeOption LinesToScrollOption;
 	ZLIntegerRangeOption LinesToKeepOption;
-	ZLBooleanOption EnableTapScrollingOption;
-	ZLBooleanOption TapScrollingOnFingerOnlyOption;
-	ZLBooleanOption TapScrollingOnDragOnlyOption;
-	ZLBooleanOption VariableDragOption;
-	ZLIntegerRangeOption VariableDragLengthOption;
 
 	ZLBooleanOption UseSeparateBindingsOption;
 
@@ -107,6 +102,7 @@ private:
 	bool isViewFinal() const;
 
 	void showLibraryView();
+	void showLibraryView(const std::string &actionId);
 
 public:
 	shared_ptr<ZLKeyBindings> keyBindings();
@@ -184,6 +180,7 @@ friend class ShowBookInfoAction;
 //friend class ScrollToStartOfTextAction;
 //friend class ScrollToEndOfTextAction;
 friend class UndoAction;
+friend class BookViewOption;
 //friend class RedoAction;
 friend class SearchAction;
 friend class SearchPatternAction;
