@@ -133,21 +133,20 @@ void FBView::setCaption(const std::string &caption) {
 }
 
 bool FBView::onStylusPress(int x, int y) {
+	ZLView::onStylusPress(x, y);
+
 	if (!myTapScroller.isNull()) {
 		ZLTimeManager::Instance().removeTask(myTapScroller);
 		myTapScroller.reset();
 	}
 
-	myPressedX = x;
-	myPressedY = y;
-	myIsReleasedWithoutMotion = false;
+	setPressProcessed(false);
 
 	if (ZLTextView::onStylusPress(x, y)) {
+		setPressProcessed(true);
 		return true;
 	}
 
-	myIsReleasedWithoutMotion = true;
-	
 	if (_onStylusPress(x, y)) {
 		return true;
 	}
@@ -180,6 +179,8 @@ void FBView::TapScroller::run() {
 }
 
 bool FBView::onStylusRelease(int x, int y) {
+	ZLView::onStylusRelease(x, y);
+
 	const bool hadSelection = !selectionModel().isEmpty();
 
 	if (!myTapScroller.isNull()) {
@@ -195,30 +196,19 @@ bool FBView::onStylusRelease(int x, int y) {
 		return true;
 	}
 
-	FBReader &fbreader = FBReader::Instance();
-	myIsReleasedWithoutMotion =
-		myIsReleasedWithoutMotion && (abs(x - pressedX()) <= 5) && (abs(y - pressedY()) <= 5);
-
-	// Do scrolling by tapping
-	if (!hadSelection && isReleasedWithoutMotion() &&
-			fbreader.EnableTapScrollingOption.value() &&
-			!fbreader.TapScrollingOnDragOnlyOption.value() &&
-			(!ZLBooleanOption(ZLCategoryKey::EMPTY, ZLOption::PLATFORM_GROUP, ZLOption::FINGER_TAP_DETECTABLE, false).value() ||
-			 !fbreader.TapScrollingOnFingerOnlyOption.value())) {
-		myTapScroller = new TapScroller(*this, y);
-		ZLTimeManager::Instance().addAutoRemovableTask(myTapScroller, doubleClickDelay());
+	// Do scrolling by dragging
+	if (isShortVerticalSwipe()) {
+	    doLineScrolling(y<pressedY());
+		return true;
+	} else if (isLongVerticalSwipe()) {
+	    doTapScrolling(y>pressedY()? 0:context().height());
 		return true;
 	}
 
-	// Do scrolling by dragging
-	int dx = x!=pressedX()? abs(x - pressedX()) : 1;
-	int dy = abs(y - pressedY());
-	if((dy / dx >= 5) && fbreader.TapScrollingOnDragOnlyOption.value()) {
-		if (fbreader.VariableDragOption.value() && (dy < fbreader.VariableDragLengthOption.value())) {
-		    doLineScrolling(y<pressedY());
-		} else {
-		    doTapScrolling(y>pressedY()? 0:context().height());
-		}
+	// Do scrolling by tapping
+	if ((!hadSelection) && isTap()) {
+		myTapScroller = new TapScroller(*this, y);
+		ZLTimeManager::Instance().addAutoRemovableTask(myTapScroller, doubleClickDelay());
 		return true;
 	}
 
@@ -246,11 +236,21 @@ bool FBView::_onStylusMove(int, int) {
 }
 
 bool FBView::onStylusMovePressed(int x, int y) {
-	if (myIsReleasedWithoutMotion) {
-		if ((abs(x - pressedX()) > 5) || (abs(y - pressedY()) > 5)) {
-			myIsReleasedWithoutMotion = false;
-			activateSelection(pressedX(), pressedY());
-		}
+	ZLView::onStylusMovePressed(x, y);
+
+ 	if (isReleasedWithoutMotion()) {
+ 		if (!isPressProcessed() && isLongPress() &&
+ 				FBReader::Instance().showPopupMenu()) {
+ 			setPressProcessed(true);
+ 			return true;
+ 		}
+ 
+ 		return false;
+ 	}
+ 
+	if (isSelectionEnabled() && !isPressProcessed()) {
+		setPressProcessed(true);
+		activateSelection(pressedX(), pressedY());
 	}
 
 	if (ZLTextView::onStylusMovePressed(x, y)) {
