@@ -405,8 +405,22 @@ XHTMLTagAction *XHTMLReader::addAction(const std::string &tag, XHTMLTagAction *a
 	return old;
 }
 
+void XHTMLReader::freeTagTable() {
+	if (!ourTagActions.empty()) {
+		for (std::map<std::string,XHTMLTagAction*>::iterator it = ourTagActions.begin(); it != ourTagActions.end(); it++) {
+			XHTMLTagAction *old = it->second;
+			it->second = 0;
+			delete old;
+		}
+
+		ourTagActions.clear();
+	}
+}
+
 void XHTMLReader::fillTagTable() {
 	if (ourTagActions.empty()) {
+		atexit(freeTagTable);
+
 		//addAction("html",	new XHTMLTagAction());
 		addAction("body",	new XHTMLTagBodyAction());
 		//addAction("title",	new XHTMLTagAction());
